"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useListenerProvider = exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const ListenerContext = /*#__PURE__*/(0, _react.createContext)({});

const useListenerProvider = () => (0, _react.useContext)(ListenerContext);

exports.useListenerProvider = useListenerProvider;

const ListenerProvider = props => {
  const [listeners, setListeners] = (0, _react.useState)({});

  const addEventListener = (event, callback) => {
    if (typeof event === 'string' && typeof callback === 'function') {
      const eventId = `${event}-${Date.now()}`;
      setListeners({ ...listeners,
        [eventId]: {
          name: event,
          callback
        }
      });
      return eventId;
    }

    return false;
  };

  const removeEventListener = eventId => {
    if (typeof eventId === 'string') {
      const newListeners = { ...listeners
      };
      delete newListeners[eventId];
      setListeners(newListeners);
      return true;
    }

    return false;
  };

  const removeAllListeners = () => {
    setListeners({});
  };

  const emitEvent = (event, data) => {
    Object.keys(listeners).forEach(id => {
      var _listeners$id;

      if ((listeners === null || listeners === void 0 ? void 0 : (_listeners$id = listeners[id]) === null || _listeners$id === void 0 ? void 0 : _listeners$id.name) === event) {
        listeners[id].callback(data);
      }
    });
  };
  /* shortener */


  const on = (event, callback) => {
    return addEventListener(event, callback);
  };

  const rm = eventId => {
    return removeEventListener(eventId);
  };

  const rmAll = () => {
    removeAllListeners();
  };

  const emit = (event, data) => {
    emitEvent(event, data);
  };

  const value = {
    addEventListener,
    removeEventListener,
    removeAllListeners,
    emitEvent,
    emit,
    on,
    rm,
    rmAll
  };
  return /*#__PURE__*/_react.default.createElement(ListenerContext.Provider, {
    value: value
  }, props.children);
};

var _default = ListenerProvider;
exports.default = _default;
//# sourceMappingURL=ListenerProvider.js.map