import React, { useState, createContext, useContext } from 'react';
const ListenerContext = /*#__PURE__*/createContext({});
export const useListenerProvider = () => useContext(ListenerContext);

const ListenerProvider = props => {
  const [listeners, setListeners] = useState({});

  const addEventListener = (event, callback) => {
    if (typeof event === 'string' && typeof callback === 'function') {
      const eventId = `${event}-${Date.now()}`;
      setListeners({ ...listeners,
        [eventId]: {
          name: event,
          callback
        }
      });
      return eventId;
    }

    return false;
  };

  const removeEventListener = eventId => {
    if (typeof eventId === 'string') {
      const newListeners = { ...listeners
      };
      delete newListeners[eventId];
      setListeners(newListeners);
      return true;
    }

    return false;
  };

  const removeAllListeners = () => {
    setListeners({});
  };

  const emitEvent = (event, data) => {
    Object.keys(listeners).forEach(id => {
      var _listeners$id;

      if ((listeners === null || listeners === void 0 ? void 0 : (_listeners$id = listeners[id]) === null || _listeners$id === void 0 ? void 0 : _listeners$id.name) === event) {
        listeners[id].callback(data);
      }
    });
  };
  /* shortener */


  const on = (event, callback) => {
    return addEventListener(event, callback);
  };

  const rm = eventId => {
    return removeEventListener(eventId);
  };

  const rmAll = () => {
    removeAllListeners();
  };

  const emit = (event, data) => {
    emitEvent(event, data);
  };

  const value = {
    addEventListener,
    removeEventListener,
    removeAllListeners,
    emitEvent,
    emit,
    on,
    rm,
    rmAll
  };
  return /*#__PURE__*/React.createElement(ListenerContext.Provider, {
    value: value
  }, props.children);
};

export default ListenerProvider;
//# sourceMappingURL=ListenerProvider.js.map